local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")

---------------
--- modules ---
---------------
local achievementTracker = core:GetModule("achievementTracker")
local announce = core:GetModule("announce")
local dailyTooltip = core:GetModule("dailyTooltip")
local notifications = core:GetModule("notifications")
local questsGivers = core:GetModule("questsGivers")
local informations = core:GetModule("informations")
local questsTooltip = core:GetModule("questsTooltip")
local questsTracker = core:GetModule("questsTracker")
local soundNotifications = core:GetModule("soundNotifications")
local tooltip = core:GetModule("tooltips")

-----------------------
--- Library Loading ---
-----------------------
local LCo = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-Config")
local LSM = LibStub("LibSharedMedia-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName = CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName
local pairs, type, sub, tonumber, format, unpack, IsAddOnLoaded = pairs, type, string.sub, tonumber, string.format, unpack, IsAddOnLoaded

-----------------------
--- Local variables ---
-----------------------
local partyZone = core.LC["PARTYQUESTSZONE"]
local questsAutoTrack = {[1] = LCo["Always"], [2] = LCo["Below Max Level"], [3] = LCo["Never"]}
local hideQuests = {[1] = LCo["When you've completed the quest"], [2] = LCo["When everyone in the party has completed the quest"], [3] = LCo["Never"]}
local hideObjectives = {[1] = LCo["When you've completed the objective"], [2] = LCo["When everyone in the party has completed the objective"], [3] = LCo["Never"]}
local attachSelect = {[1] = LCo["Right"], [2] = LCo["Left"], [3] = LCo["Dettached"]}
local automaticAcceptChoices = {[1] = LCo["All quests"], [2] = LCo["Only daily quests"], [3] = LCo["All but daily quests"], [4] = LCo["Never"]}
local anchorPosition = {[1] = "TOPLEFT", [2] = "TOP", [3] = "TOPRIGHT", [4] = "BOTTOMLEFT", [5] = "BOTTOM", [6] = "BOTTOMRIGHT"}
local outlines = {[""] = LCo["None"], ["OUTLINE"] = LCo["Outline"], ["THICKOUTLINE"] = LCo["Thick Outline"]}
local achievementPosition = {[1] = LCo["Above"], [2] = LCo["Below"]}
local questsInteractivityList = questsTooltip.interactivityList
local achievementInteractivityList = {
	["chatLink"] = LCo["Insert achievement link in chat"],
	["achievementTooltip"] = LCo["Display of achievement tooltip"],
	["removeTrackedAchievement"] = LCo["Remove achievement from tracker"],
	["displayAchievement"] = LCo["Diplay achievement frame"],
	["nothing"] = LCo["Do nothing"],
}
local ldbInteractivityList = {
	["openOptions"] = LCo["Open tomQuest2 option frame"],
	["toggleQuestTracker"] = LCo["Toggle quests tracking frame"],
	["toggleAchievementTracker"] = LCo["Toggle achievements tracking frame"],
	["toggleAnnouncementModule"] = LCo["toggle chat announcement module"],
	["toggleQuestsLog"] = LCo["Toggle Blizzard quests log"],
	["toggleSyncPartyModule"] = LCo["Toggle party synchronization module"],
	["toggleQuestsTrackerModule"] = LCo["Toggle quests tracking module"],
	["toggleAchievementTrackerModule"] = LCo["Toggle achievements tracking module"],
	["nothing"] = LCo["Do nothing"],
}
local moneySeparators = {
	["dot"] = LCo["Dot"],
	["symbol"] = LCo["Symbol"],
	["texture"] = LCo["Texture"],
}
local questLevelTag = {
	["questLevel"] = LCo["Quest level"],
	["weightedQuestLevel"] = LCo["Weighted quest level"],
	["Hide"] = LCo["Hide"],
}
local raidModeValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
}
local raidModeTrackerValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
	["hiddenInRaid"] = LCo["Hidden once entering a raid"],
	["hiddenInRaidInstance"] = LCo["Hidden once entering a raid instance (will work in pvp and outdoor raids)"],
}
local zSorting = {
	["levelA"] = LCo["Sort zones based on level (ascending)"],
	["levelD"] = LCo["Sort zones based on level (descending)"],
	["nameA"] = LCo["Sort zones based on zone name (ascending)"],
	["nameD"] = LCo["Sort zones based on zone name (descending)"],
}
local zwlPosition = {
	[0] = LCo["Zones without level first"],
	[100] = LCo["Zones without level last"]
}
local qSorting = {
	["weightedA"] = LCo["Sort quests based on weighted level (ascending)"],
	["weightedD"] = LCo["Sort quests based on weighted level (descending)"],
	["levelA"] = LCo["Sort quests based on quest level (ascending)"],
	["levelD"] = LCo["Sort quests based on quest level (descending)"],
	["nameA"] = LCo["Sort quests based on name (ascending)"],
	["nameD"] = LCo["Sort quests based on name (descending)"],
}

-----------------------
--- Tools functions ---
-----------------------
local function normalizeWidth(min, max)
	if max < min then return max else return min end
end
local function computeQuestsObjectivesStatus()
	for uid, qoData in pairs (core.questsObjectives) do
		for objective in pairs (qoData) do
			core:computeQuestObjectiveStatus(uid, objective)
		end
	end
	if core.moduleState["questsTracker"] then
		questsTracker:ReleaseTracker()
		questsTracker:updateQuestsTracker()
	end
	core.cbh:Fire("questLogUpdated", true, true)
end
local function normalizeColorComponent(component)
	if component > 1 then return 1 elseif component < 0 then return 0 else return component end
end

---------------------------------------------
--- Core option tables & get set function ---
---------------------------------------------
local function getLBF(info)
	return core.db.profile.skin[info[#info]]
end
local function setSkin(info, value)
	core.LBFGroup:Skin(value, core.LBFGroup.Gloss, core.LBFGroup.Backdrop)
end
local function setGloss(info, value)
	core.LBFGroup:Skin(core.LBFGroup.SkinID, value, core.LBFGroup.Backdrop)
end
local function setBackdrop(info, value)
	core.LBFGroup:Skin(core.LBFGroup.SkinID, core.LBFGroup.Gloss, value and true or false)
end
local function getLayerColor(info)
	return core.LBFGroup:GetLayerColor(info[#info])
end
local function setLayerColor(info, r,g,b,a)
	core.LBFGroup:Skin(core.LBFGroup.SkinID, core.LBFGroup.Gloss, core.LBFGroup.Backdrop, info[#info], r, g, b, a)
end
local function resetColors(info)
	core.LBFGroup:ResetColors()
end
local function optMGetter(info)
	return core:getModuleEnabled(info[#info])
end
local function optMSetter(info, value)
	core:setModuleEnabled(info[#info], value)
	if info[#info] == "questsTracker" then
		if not value then
			core.cbh:Fire("questsTrackerDisabled")
		else
			core.cbh:Fire("questsTrackerEnabled")
		end
	elseif info[#info] == "questsTooltip" and value then
		core.cbh:Fire("questLogUpdated", true)
	end
end
local function optRMGetter(info)
	return core.db.profile.raidModules[info[#info]]
end
local function optRMSetter(info, value)
	local oldState = core.db.profile.raidModules.enabled
	core.db.profile.raidModules[info[#info]] = value
	if core.db.profile.raidModules.enabled then
		core:checkRaid(nil, true)
	elseif oldState then
		for k in core:IterateModules() do
			if core.db.profile.modules[k] ~= core.moduleState[k] and not core.moduleState[k] then
				core:EnableModule(k)
				core.moduleState[k] = true
				core:Print(format(LCo["%s module is now Enabled"], k))
			end
			if core.db.profile.raidModules[k] == "hiddenInRaid" or core.db.profile.raidModules[k] == "hiddenInRaidInstance" then
				if k == "questsTracker" then
					tQ2toggleQuestTracker(1)
				elseif k == "achievementTracker" then
					tQ2toggleAchievementTracker(1)
				end
			end
		end
	end
end
local function optQGetter(info)
	return core.db.profile.quests[info[#info]]
end
local function optQSetter(info, value)
	core.db.profile.quests[info[#info]] = value
end
local function partyHeaderSetter(info, value)
	core.db.profile.quests[info[#info]] = value
	if value then
		if core.numPartyMembers == 0 then return end
		for uid in pairs(core.quests) do
			if not core.quests[uid].complete[core.playerName] then
				local zone = core.quests[uid].zone
				core.zones[zone].numQuests = core.zones[zone].numQuests - 1
				if core.zones[zone].numQuests == 0 then
					wipe(core.zones[zone])
					core.zones[zone] = nil
					wipe(core.questsByZone[zone])
					core.questsByZone[zone] = nil
					wipe(core.sortedQuestsByZone[zone])
					core.sortedQuestsByZone[zone] = nil
				end
				if not core.zones[partyZone] then
					core.addZone(partyZone)
				end
				core.questsByZone[partyZone][uid] = true
				core.zones[partyZone].numQuests = core.zones[partyZone].numQuests + 1
			end
		end
		core.zonesSort()
		core.questsSort()
		if core.moduleState["questsTracker"] then
			questsTracker:ReleaseTracker()
			questsTracker:updateQuestsTracker()
		end
	else
		if core.numPartyMembers == 0 or not core.questsByZone[partyZone] then return end
		for uid in pairs(core.questsByZone[partyZone]) do
			local zone = core.quests[uid].zone
			if not core.zones[zone] then
				core.addZone(zone)
			end
			core.questsByZone[zone][uid] = true
			core.zones[zone].numQuests = core.zones[zone].numQuests + 1
			if zone == core.localClass then
				core.quests[uid].class = true
			end
		end
		wipe(core.zones[partyZone])
		core.zones[partyZone] = nil
		wipe(core.questsByZone[partyZone])
		core.questsByZone[partyZone] = nil
		wipe(core.sortedQuestsByZone[partyZone])
		core.sortedQuestsByZone[partyZone] = nil
		core.zonesSort()
		core.questsSort()
		if core.moduleState["questsTracker"] then
			questsTracker:ReleaseTracker()
			questsTracker:updateQuestsTracker()
		end
	end
end
local function optQMSetter(info, value)
	core.db.profile.quests[info[#info]] = value
	core.cbh:Fire("updateAchievementTracker")
end
local function hexColorGetter(info)
	return tonumber(sub(core.db.profile.quests[info[#info]], 1, 2), 16) / 255, tonumber(sub(core.db.profile.quests[info[#info]], 3, 4), 16) / 255, tonumber(sub(core.db.profile.quests[info[#info]], 5, 6), 16) / 255
end
local function hexColorSetter(info, arg1, arg2, arg3)
	core.db.profile.quests[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
	core.completedString = "|cff"..core.db.profile.quests.completedColor..core.LC["Finished"].."|r"
	core.failedString = "|cff"..core.db.profile.quests.failedColor..core.LC["Failed"].."|r"
	for uid in pairs (core.quests) do
		core:computeQuestStatus(uid)
	end
	if core.moduleState["questsTracker"] then
		questsTracker:ReleaseTracker()
		questsTracker:updateQuestsTracker()
	end
	core.cbh:Fire("questLogUpdated", true, true)
end
local function optWQLSetter(info, value)
	core.db.profile.quests[info[#info]] = value
	core:computeQuestLevel()
end
local function optCCFSetter(info, value)
	core.db.profile.quests[info[#info]] = value
	computeQuestsObjectivesStatus()
end
local function optTagSetter(info, value)
	core.db.profile.quests[info[#info]] = value
	core:computeQuestTag()
end
local function sortSetter(info, value)
	core.db.profile.quests[info[#info]] = value
	core.zonesSort()
	core.questsSort()
end
local function optHideBTr(info, value)
	core.db.profile.quests[info[#info]] = value
	core:hideShowDefaultBlizTracker()
end
local function specialItemMacro(info, value)
	local index = GetMacroIndexByName("tomQuest2_SI")
	if index == 0 then
		index = CreateMacro("tomQuest2_SI", 1, "", 0)
		PickupMacro(index)
	else
		EditMacro(index, "tomQuest2_SI", 1, "")
	end
end
local function hideCustomColorMode()
	if core.db.profile.quests.customColorMode then
		return true
	else
		return false
	end
end
local function zeroColorGetter(info)
	return normalizeColorComponent(core.db.profile.quests.rj0) , normalizeColorComponent(core.db.profile.quests.gj0), normalizeColorComponent(core.db.profile.quests.bj0)
end
local function halfColorGetter(info)
	return normalizeColorComponent(0.5*core.db.profile.quests.ri1 + core.db.profile.quests.rj1), normalizeColorComponent(0.5*core.db.profile.quests.gi1 + core.db.profile.quests.gj1), normalizeColorComponent(0.5*core.db.profile.quests.bi1 + core.db.profile.quests.bj1)
end
local function fullColorGetter(info)
	return normalizeColorComponent(core.db.profile.quests.rj3), normalizeColorComponent(core.db.profile.quests.gj3), normalizeColorComponent(core.db.profile.quests.bj3)
end
local function computeFirstFunction(r0, g0, b0, rh, gh, bh)
	core.db.profile.quests.rj1 = r0
	core.db.profile.quests.gj1 = g0
	core.db.profile.quests.bj1 = b0
	core.db.profile.quests.ri1 = 2 * (rh - r0)
	core.db.profile.quests.gi1 = 2 * (gh - g0)
	core.db.profile.quests.bi1 = 2 * (bh - b0)
end
local function computeSecondFunction(rh, gh, bh, r1, g1, b1)
	core.db.profile.quests.rj2 = 2 * rh - r1
	core.db.profile.quests.gj2 = 2 * gh - g1
	core.db.profile.quests.bj2 = 2 * bh - b1
	core.db.profile.quests.ri2 = 2 * (r1 - rh)
	core.db.profile.quests.gi2 = 2 * (g1 - gh)
	core.db.profile.quests.bi2 = 2 * (b1 - bh)
end
local function zeroColorSetter(info, arg1, arg2, arg3)
	core.db.profile.quests.rj0 = arg1
	core.db.profile.quests.gj0 = arg2
	core.db.profile.quests.bj0 = arg3
	local rh, gh, bh = halfColorGetter()
	computeFirstFunction(arg1, arg2, arg3, rh, gh, bh)
	computeQuestsObjectivesStatus()
end
local function halfColorSetter(info, arg1, arg2, arg3)
	local r0, g0, b0 = zeroColorGetter()
	computeFirstFunction(r0, g0, b0, arg1, arg2, arg3)
	local r1, g1, b1 = fullColorGetter()
	computeSecondFunction(arg1, arg2, arg3, r1, g1, b1)
	computeQuestsObjectivesStatus()
end
local function fullColorSetter(info, arg1, arg2, arg3)
	local rh, gh, bh = halfColorGetter()
	computeSecondFunction(rh, gh, bh, arg1, arg2, arg3)
	core.db.profile.quests.rj3 = arg1
	core.db.profile.quests.gj3 = arg2
	core.db.profile.quests.bj3 = arg3
	computeQuestsObjectivesStatus()
end
local function setDefaultColor()
	core.db.profile.quests.rj0, core.db.profile.quests.gj0, core.db.profile.quests.bj0 = 1, 0, 0
	core.db.profile.quests.ri1, core.db.profile.quests.rj1, core.db.profile.quests.gi1, core.db.profile.quests.gj1, core.db.profile.quests.bi1, core.db.profile.quests.bj1 = 0, 1, 2, 0, 0 ,0
	core.db.profile.quests.ri2, core.db.profile.quests.rj2, core.db.profile.quests.gi2, core.db.profile.quests.gj2, core.db.profile.quests.bi2, core.db.profile.quests.bj2 = -2, 2, 0, 1, 0, 0
	core.db.profile.quests.rj3, core.db.profile.quests.gj3, core.db.profile.quests.bj3 = 0, 1, 0
	computeQuestsObjectivesStatus()
end
local function zwlSetter(info, value)
	core.db.profile.quests[info[#info]] = value
	for zone, zData in pairs(core.zones) do
		if zData.minLevel == "" then
			core.zones[zone].sMinLevel = core.db.profile.quests.zoneWithoutLvlPos
		end
		if zData.maxLevel == "" then
			core.zones[zone].sMaxLevel = core.db.profile.quests.zoneWithoutLvlPos
		end
		core.zonesSort()
	end
end
local function hideRaidMode()
	if core.db.profile.raidModules.enabled then
		return false
	else
		return true
	end
end

---------------------------------------
--- Achievement get & set functions ---
---------------------------------------
local function achievementOptGetter(info)
	return achievementTracker.db.profile[info[#info]]
end
local function achievementOptSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	achievementTracker:ReleaseTracker()
	achievementTracker:updateAchievementTracker()
end
local function achievementLayoutSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] or achievementTracker.db.profile.showInQuestTracker then return end
	achievementTracker:initTrackerSettings(achievementTracker.db.profile.disableInteraction, achievementTracker.db.profile.maxWidth, achievementTracker.db.profile.statusMaxWidth, achievementTracker.db.profile.minWidth, achievementTracker.db.profile.statusMinWidth)
	achievementTracker:ReleaseTracker()
	achievementTracker:updateAchievementTracker()
end
local function achievementInQuestsTrackerSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	achievementTracker:ReleaseTracker()
	if not value then
		achievementTracker:initTrackerSettings(achievementTracker.db.profile.disableInteraction, achievementTracker.db.profile.maxWidth, achievementTracker.db.profile.statusMaxWidth, achievementTracker.db.profile.minWidth, achievementTracker.db.profile.statusMinWidth)
		if core.moduleState["questsTracker"] then
			questsTracker:ReleaseTracker()
			questsTracker:updateQuestsTracker()
		end
	elseif core.moduleState["questsTracker"] then
		questsTracker:initTrackerSettings()
		questsTracker:ReleaseTracker()
		questsTracker:updateQuestsTracker()
	end
end
local function timedAchievementSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	achievementTracker.setTimedAchievements()
end
local function achievementColorGetter(info)
	return unpack(achievementTracker.db.profile[info[#info]])
end
local function achievementColorSetter(info, r, g, b, a)
	achievementTracker.db.profile[info[#info]][1] = r
	achievementTracker.db.profile[info[#info]][2] = g
	achievementTracker.db.profile[info[#info]][3] = b
	achievementTracker.db.profile[info[#info]][4] = a
	if not core.moduleState["achievementTracker"] then return end
	achievementTracker:updateAchievementTracker()
end
local function achievementHexColorGetter(info)
	return tonumber(sub(achievementTracker.db.profile[info[#info]], 1, 2), 16) / 255, tonumber(sub(achievementTracker.db.profile[info[#info]], 3, 4), 16) / 255, tonumber(sub(achievementTracker.db.profile[info[#info]], 5, 6), 16) / 255
end
local function achievementHexColorSetter(info,r, g, b)
	if not core.moduleState["achievementTracker"] then return end
	achievementTracker.db.profile[info[#info]] = format("%02x%02x%02x", r*255, g*255, b*255)
	achievementTracker:updateAchievementTracker()
end
local function achievementFontSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	achievementTracker.setFonts()
	achievementTracker:updateAchievementTracker()
end
local function achievementAnchorSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	if value then
		achievementTracker.anchor:Show()
	else
		achievementTracker.anchor:Hide()
	end
	achievementTracker:updateAchievementTracker()
end
local function achievementCombatSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	achievementTracker.setTrackerInCombat()
	achievementTracker.hideInCombat = nil
	achievementTracker:updateAchievementTracker()
end
local function achievementUnitTooltipSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	if value then
		achievementTracker.setUnitTooltip()
	else
		wipe(core.achievementsMobs)
		for _, aid in pairs(achievementTracker.achMobs) do
			core.achievementName[aid] = nil
			core.achievementIcon[aid] = nil
		end
	end
end
local function achievementItemTooltipSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	if value then
		achievementTracker.setItemTooltip()
	else
		wipe(core.achievementsItems)
		for _, aid in pairs(achievementTracker.achItems) do
			core.achievementName[aid] = nil
			core.achievementIcon[aid] = nil
		end
	end
end
local function achievementTradeSkillsIUSetter(info, value)
	achievementTracker.db.profile[info[#info]] = value
	if not core.moduleState["achievementTracker"] then return end
	if value then
		achievementTracker.setTSUI()
	else
		wipe(achievementTracker.achievementsTradeSkills)
		for _, aid in pairs(achievementTracker.achTrade) do
			core.achievementName[aid] = nil
			core.achievementIcon[aid] = nil
		end
		if IsAddOnLoaded("Blizzard_TradeSkillUI") then
			achievementTracker:Unhook("TradeSkillFrame_Update")
		elseif IsAddOnLoaded("MrTrader_SkillWindow") then
			achievementTracker:Unhook(MRTSkillWindow, "UpdateSkillList")
		elseif IsAddOnLoaded("Skillet") then
			achievementTracker:Unhook(Skillet, "internal_UpdateTradeSkillWindow")
		elseif IsAddOnLoaded("AdvancedTradeSkillWindow") then
			achievementTracker:Unhook("ATSWFrame_Update")
		else
			achievementTracker:UnregisterEvent("ADDON_LOADED")
		end
	end
end
local function achievementInteractionDisabled()
	if achievementTracker.db.profile.disableInteraction then
		return true
	else
		return false
	end
end
------------------------------------
--- Announce get & set functions ---
------------------------------------
local function announceOptGetter(info)
	return announce.db.profile[info[#info]]
end
local function announceOptSetter(info, value)
	announce.db.profile[info[#info]] = value
	if not core.moduleState["announce"] then return end
	if announce.callBacks[info[#info]] then
		if value then
			core.RegisterCallback(announce, info[#info])
		else
			core.UnregisterCallback(announce, info[#info])
		end
	end
end
----------------------------------------
--- DailyTooltip get & set functions ---
----------------------------------------
local function dailyTooltipOptGetter(info)
	return dailyTooltip.db.profile[info[#info]]
end
local function dailyTooltipOptSetter(info, value)
	dailyTooltip.db.profile[info[#info]] = value
end
local function dailyTooltipDailySetter(info, value)
	dailyTooltip.db.profile[info[#info]] = value
	if not core.moduleState["dailyTooltip"] then return end
	dailyTooltip:updateLDBText()
end
local function dailyTooltipTimeSetter(info, value)
	dailyTooltip.db.profile[info[#info]] = value
	if not core.moduleState["dailyTooltip"] then return end
	dailyTooltip:SetTimer()
	dailyTooltip:updateLDBText()
end
local function dailyTooltipFontSetter(info, value)
	dailyTooltip.db.profile[info[#info]] = value
	if not core.moduleState["dailyTooltip"] then return end
	dailyTooltip.DtFontTQ2:SetFont(LSM:Fetch("font", dailyTooltip.db.profile.font), dailyTooltip.db.profile.fontSize, dailyTooltip.db.profile.fontOutline)
end
-----------------------------------------
--- Notifications get & set functions ---
-----------------------------------------
local function notificationsOptGetter(info)
	return notifications.db.profile[info[#info]]
end
local function notificationsOptSetter(info, value)
	notifications.db.profile[info[#info]] = value
	if not core.moduleState["notifications"] then return end
end
local function notificationsCallbackSetter(info, value)
	notifications.db.profile[info[#info]] = value
	if not core.moduleState["notifications"] then return end
	if value then
		core.RegisterCallback(notifications, info[#info])
	else
		core.UnregisterCallback(notifications, info[#info])
	end
end
local function notificationsHideBlizzardNotificationSetter(info, value)
	notifications.db.profile[info[#info]] = value
	if not core.moduleState["notifications"] then return end
	notifications:SetBlizzNotification()
end
----------------------------------------
--- QuestsGivers get & set functions ---
----------------------------------------
local function questsGiversOptGetter(info)
	return questsGivers.db.profile[info[#info]]
end
local function questsGiversOptSetter(info, value)
	questsGivers.db.profile[info[#info]] = value
end
----------------------------------------------
--- QuestsInformations get & set functions ---
----------------------------------------------
local function questsInformationsOptGetter(info)
	return informations.db.profile[info[#info]]
end
local function questsInformationsOptSetter(info, value)
	informations.db.profile[info[#info]] = value
end
local function questsInformations(info, value)
	informations.db.profile[info[#info]] = value
	if not core.moduleState["informations"] then return end
	local uid
	if tomQuest2QlFrame then
		uid = tomQuest2QlFrame.currentUid
		tomQuest2QlFrame.currentUid = nil
	end
	if tomQuest2LhFrame then
		uid = tomQuest2LhFrame.currentUid or uid
		tomQuest2LhFrame.currentUid = nil
	end
	if uid then
		informations:displayQuestInformations(nil, uid)
	end
end
local function questsInformationsColorGetter(info)
	return unpack(informations.db.profile[info[#info]])
end
local function questsInformationsColorSetter(info, arg1, arg2, arg3, arg4)
	informations.db.profile[info[#info]][1] = arg1
	informations.db.profile[info[#info]][2] = arg2
	informations.db.profile[info[#info]][3] = arg3
	informations.db.profile[info[#info]][4] = arg4
	if not core.moduleState["informations"] then return end
	local uid
	if tomQuest2ParentFrame then
		tomQuest2ParentFrame:SetBackdropColor(informations.db.profile.backDropColor[1], informations.db.profile.backDropColor[2], informations.db.profile.backDropColor[3], informations.db.profile.backDropColor[4])
		tomQuest2ParentFrame:SetBackdropBorderColor(informations.db.profile.borderColor[1], informations.db.profile.borderColor[2], informations.db.profile.borderColor[3], informations.db.profile.borderColor[4])
	end
	if tomQuest2QlFrame then
		tomQuest2QlFrame:SetBackdropColor(informations.db.profile.backDropColor[1], informations.db.profile.backDropColor[2], informations.db.profile.backDropColor[3], informations.db.profile.backDropColor[4])
		tomQuest2QlFrame:SetBackdropBorderColor(informations.db.profile.borderColor[1], informations.db.profile.borderColor[2], informations.db.profile.borderColor[3], informations.db.profile.borderColor[4])
		informations:lockUnlockQlFrame()
		uid = tomQuest2QlFrame.currentUid
		tomQuest2QlFrame.currentUid = nil
	end
	if tomQuest2LhFrame then
		tomQuest2LhFrame:SetBackdropColor(informations.db.profile.backDropColor[1], informations.db.profile.backDropColor[2], informations.db.profile.backDropColor[3], informations.db.profile.backDropColor[4])
		tomQuest2LhFrame:SetBackdropBorderColor(informations.db.profile.borderColor[1], informations.db.profile.borderColor[2], informations.db.profile.borderColor[3], informations.db.profile.borderColor[4])
		informations:lockUnlockLhFrame()
		uid = tomQuest2LhFrame.currentUid or uid
		tomQuest2LhFrame.currentUid = nil
	end
	if uid then
		informations:displayQuestInformations(nil, uid)
	end
end
local function questsInformationsFrameSetter(info, value)
	informations.db.profile[info[#info]] = value
	if not core.moduleState["informations"] then return end
	informations:setFrameLayout()
end
-----------------------------------------
--- QuestsTooltip get & set functions ---
-----------------------------------------
local function questsTooltipOptGetter(info)
	return questsTooltip.db.profile[info[#info]]
end
local function questsTooltipOptSetter(info, value)
	questsTooltip.db.profile[info[#info]] = value
end
local function questsTooltipLayoutSetter(info, value)
	questsTooltip.db.profile[info[#info]] = value
	if not core.moduleState["questsTooltip"] then return end
	questsTooltip.collapseButtonSize = questsTooltip.db.profile.showCollapseButton and questsTooltip.db.profile.collapseButtonSize or 0
	questsTooltip.maxWidth = questsTooltip.db.profile.maxWidth
	questsTooltip.minWidth = normalizeWidth(questsTooltip.db.profile.minWidth, questsTooltip.maxWidth)
	questsTooltip.statusMaxWidth = questsTooltip.db.profile.statusMaxWidth
	questsTooltip.statusMinWidth = normalizeWidth(questsTooltip.db.profile.statusMinWidth, questsTooltip.statusMaxWidth)
	questsTooltip.totalMaxWidth = questsTooltip.maxWidth + questsTooltip.statusMaxWidth + questsTooltip.CELL_MARGIN_H
	questsTooltip.totalMinWidth = normalizeWidth(questsTooltip.minWidth + questsTooltip.statusMinWidth + questsTooltip.CELL_MARGIN_H, questsTooltip.totalMaxWidth)
	if questsTooltip.db.profile.showQuestItemIcon then
		questsTooltip.questItemIconSize = questsTooltip.db.profile.questItemIconSize
		questsTooltip.questsCellPadding = questsTooltip.db.profile.questsCellPadding + questsTooltip.questItemIconSize
		questsTooltip.objectivesCellPadding = questsTooltip.db.profile.objectivesCellPadding + questsTooltip.questItemIconSize
	else
		questsTooltip.questItemIconSize = 0
		questsTooltip.questsCellPadding = questsTooltip.db.profile.questsCellPadding
		questsTooltip.objectivesCellPadding = questsTooltip.db.profile.objectivesCellPadding
	end
end
local function questsTooltipColorGetter(info)
	return unpack(questsTooltip.db.profile[info[#info]])
end
local function questsTooltipColorSetter(info, arg1, arg2, arg3, arg4)
	questsTooltip.db.profile[info[#info]][1] = arg1
	questsTooltip.db.profile[info[#info]][2] = arg2
	questsTooltip.db.profile[info[#info]][3] = arg3
	questsTooltip.db.profile[info[#info]][4] = arg4
end
local function questsTooltipHexColorGetter(info)
	return tonumber(sub(questsTooltip.db.profile[info[#info]], 1, 2), 16) / 255, tonumber(sub(questsTooltip.db.profile[info[#info]], 3, 4), 16) / 255, tonumber(sub(questsTooltip.db.profile[info[#info]], 5, 6), 16) / 255
end
local function questsTooltipHexColorSetter(info, arg1, arg2, arg3)
	questsTooltip.db.profile[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
end
local function questsTooltipLdbTextSetter(info, value)
	questsTooltip.db.profile[info[#info]] = value
	if not core.moduleState["questsTooltip"] then return end
	questsTooltip:questLogUpdated(nil, true, false)
end
local function questsTooltipFontSetter(info, value)
	questsTooltip.db.profile[info[#info]] = value
	if not core.moduleState["questsTooltip"] then return end
	questsTooltip.setFont()
end
-----------------------------------------
--- QuestsTracker get & set functions ---
-----------------------------------------
local function questsTrackerOptGetter(info)
	return questsTracker.db.profile[info[#info]]
end
local function questsTrackerOptSetter(info, value)
	questsTracker.db.profile[info[#info]] = value
	if not core.moduleState["questsTracker"] then return end
	questsTracker:ReleaseTracker()
	questsTracker:updateQuestsTracker()
end
local function questsTrackerLayoutSetter(info, value)
	questsTracker.db.profile[info[#info]] = value
	if not core.moduleState["questsTracker"] then return end
	questsTracker:ReleaseTracker()
	questsTracker:initTrackerSettings()
	questsTracker:updateQuestsTracker()
end
local function questsTrackerColorGetter(info)
	return unpack(questsTracker.db.profile[info[#info]])
end
local function questsTrackerColorSetter(info, arg1, arg2, arg3, arg4)
	questsTracker.db.profile[info[#info]][1] = arg1
	questsTracker.db.profile[info[#info]][2] = arg2
	questsTracker.db.profile[info[#info]][3] = arg3
	questsTracker.db.profile[info[#info]][4] = arg4
	if not core.moduleState["questsTracker"] then return end
	questsTracker:ReleaseTracker()
	questsTracker:updateQuestsTracker()
end
local function questsTrackerHexColorGetter(info)
	return tonumber(sub(questsTracker.db.profile[info[#info]], 1, 2), 16) / 255, tonumber(sub(questsTracker.db.profile[info[#info]], 3, 4), 16) / 255, tonumber(sub(questsTracker.db.profile[info[#info]], 5, 6), 16) / 255
end
local function questsTrackerHexColorSetter(info, arg1, arg2, arg3)
	questsTracker.db.profile[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
	if not core.moduleState["questsTracker"] then return end
	questsTracker:ReleaseTracker()
	questsTracker:updateQuestsTracker()
end
local function questsTrackerAnchorSetter(info, value)
	questsTracker.db.profile[info[#info]] = value
	if not core.moduleState["questsTracker"] then return end
	if value then
		questsTracker.anchor:Show()
	else
		questsTracker.anchor:Hide()
	end
	questsTracker:updateQuestsTracker()
end
local function questsTrackerCombatSetter(info, value)
	questsTracker.db.profile[info[#info]] = value
	if not core.moduleState["questsTracker"] then return end
	questsTracker.hideInCombat = nil
	questsTracker.setTrackerInCombat()
end
local function questsTrackerMaxLevelDifferenceGetter(info)
	return core.db.profile.quests[info[#info]]
end
local function questsTrackerMaxLevelDifferenceSetter(info, value)
	core.db.profile.quests[info[#info]] = value
	if not core.moduleState["questsTracker"] then return end
	questsTracker:ReleaseTracker()
	questsTracker:updateQuestsTracker()
end
local function questsTrackerFontSetter(info, value)
	questsTracker.db.profile[info[#info]] = value
	if not core.moduleState["questsTracker"] then return end
	questsTracker.setFont()
	questsTracker:ReleaseTracker()
	questsTracker:updateQuestsTracker()
end
local function questsTrackerQuestFontSetter(info, value)
	questsTracker.db.profile[info[#info]] = value
	if not core.moduleState["questsTracker"] then return end
	questsTracker.qTrFontTQ2:SetFont(LSM:Fetch("font", questsTracker.db.profile.questFont), questsTracker.db.profile.questFontSize, questsTracker.db.profile.questFontOutline)
	questsTracker:ReleaseTracker()
	questsTracker:updateQuestsTracker()
end
local function questsTrackerQuestObjectiveFontSetter(info, value)
	questsTracker.db.profile[info[#info]] = value
	if not core.moduleState["questsTracker"] then return end
	questsTracker.qoTrFontTQ2:SetFont(LSM:Fetch("font", questsTracker.db.profile.questObjectiveFont), questsTracker.db.profile.questObjectiveFontSize, questsTracker.db.profile.questObjectiveFontOutline)
	questsTracker:ReleaseTracker()
	questsTracker:updateQuestsTracker()
end
local function questsTrackerInteractionDisabled()
	if questsTracker.db.profile.disableInteraction then
		return true
	else
		return false
	end
end
----------------------------------------------
--- SoundNotifications get & set functions ---
----------------------------------------------
local function soundNotificationsOptGetter(info)
	return soundNotifications.db.profile[info[#info]]
end
local function soundNotificationsOptSetter(info, value)
	soundNotifications.db.profile[info[#info]] = value
end
--------------------------------------------------
--- Tooltips get & set functions ---
--------------------------------------------------
local function tooltipOptGetter(info)
	return tooltip.db.profile[info[#info]]
end
local function tooltipOptSetter(info, value)
	tooltip.db.profile[info[#info]] = value
end
------------------------------
--- tomQuest2 option setup ---
------------------------------
local options = {
	type = "group",
	childGroups = "tab",
	name = "tomQuest2",
	get = optQGetter,
	set = optQSetter,
	args = {
		modules = {
			type = "group",
			name = LCo["Modules"],
			get = optMGetter,
			set = optMSetter,
			order = 1,
			args = {
			},
		},
		raidMode = {
			type = "group",
			name = LCo["Raid mode"],
			get = optRMGetter,
			set = optRMSetter,
			order = 2,
			args = {
				enabled = {
					type = "toggle",
					name = LCo["Raid mode"],
					width = "full",
					order = 1,
				},
				raidModeDesc = {
					type = "description",
					name = LCo["Define module state when in a raid"],
					order = 2,
				},
			}
		},
		other = {
			type = "group",
			name = LCo["Other"],
			order = 3,
			args = {
				partyHeader = {
					name = LCo["Show all party quests you don't have under a specific header"],
					type = "toggle",
					set = partyHeaderSetter,
					order = 1,
					width = "full",
				},
				colorizeMoney = {
					name = LCo["Color money"],
					type= "toggle",
					set = optQMSetter,
					order = 2,
					width = "full",
				},
				moneySeparator = {
					name = LCo["Money separator"],
					type= "select",
					set = optQMSetter,
					values = moneySeparators,
					order = 3,
					width = "full",
				},
				hideDefaultBlizTracker = {
					name = LCo["Hide default blizzard quests and achievement tracker"],
					type= "toggle",
					set = optHideBTr,
					order = 4,
					width = "full",
				},
				macro = {
					name = LCo["Macro Generator"],
					type = "execute",
					func = specialItemMacro,
					order = 5,
				},
			},
		},
		weightedLevel = {
			type = "group",
			name = LCo["Weighted quest level computation"],
			set = optWQLSetter,
			order = 4,
			args = {
				eliteAddLevel = {
					name = LCo["Level to add for an elite quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 1,
				},
				groupAddLevel = {
					name = LCo["Level to add per missing player for a group quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 2,
				},
				dungeonAddLevel = {
					name = LCo["Level to add per missing player for a dungeon quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 3,
				},
				heroicAddLevel = {
					name = LCo["Level to add per missing player for an heroic quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 4,
				},
				raidAddLevel = {
					name = LCo["Level to add per missing player for a raid quest"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					width = "full",
					order = 5,
				},
			},
		},
		questsTag = {
			type = "group",
			name = LCo["Custom quests tag"],
			set = optTagSetter,
			order = 5,
			args = {
				level = {
					name = LCo["Quest level tag"],
					type = "select",
					values = questLevelTag,
					width = "full",
					order = 1,
				},
				elite = {
					name = LCo["Elite quest tag"],
					type = "input",
					order = 2,
				},
				group = {
					name = LCo["Group quest tag"],
					type = "input",
					order = 3,
				},
				pvp = {
					name = LCo["PVP quest tag"],
					type = "input",
					order = 4,
				},
				raid = {
					name = LCo["Raid quest tag"],
					type = "input",
					order = 5,
				},
				dungeon = {
					name = LCo["Dungeon quest tag"],
					type = "input",
					order = 6,
				},
				heroic = {
					name = LCo["Heroic quest tag"],
					type = "input",
					order = 7,
				},
				daily = {
					name = LCo["Daily quest tag"],
					type = "input",
					order = 8,
				},
				escort = {
					name = LCo["Escort quest tag"],
					type = "input",
					order = 9,
				},
			},
		},
		customColor = {
			type = "group",
			name = LCo["Custom objective coloring function"],
			set = optCCFSetter,
			order = 6,
			args = {
				easyColorSetupDesc = {
					type = "description",
					name = LCo["You can define a color for percentage of completion = 0, 50% and 100%. tomQuest2 will automatically define the needed color function between those threee points"],
					width = "full",
					order = 0,
				},
				zeroColor = {
					name = LCo["Color for completion = 0%"],
					type = "color",
					hasAlpha = false,
					get = zeroColorGetter,
					set = zeroColorSetter,
					width = "full",
					order = 1,
				},
				halfColor = {
					name = LCo["Color for completion = 50%"],
					type = "color",
					hasAlpha = false,
					get = halfColorGetter,
					set = halfColorSetter,
					width = "full",
					order = 2,
				},
				fullColor = {
					name = LCo["Color for completion = 100%"],
					type = "color",
					hasAlpha = false,
					get = fullColorGetter,
					set = fullColorSetter,
					width = "full",
					order = 3,
				},
				resetColor = {
					name = LCo["Reset color functions"],
					type = "execute",
					func = setDefaultColor,
					width = "full",
					order = 4,
				},
				customColorMode = {
					name = LCo["Hide custom color function setup"],
					type = "toggle",
					get = optQGetter,
					set = optQSetter,
					width = "full",
					order = 5,
				},
				customColorFunctionDesc = {
					type = "description",
					name = LCo["Custom objective coloring allows you to define multiplicative and additive component of the coloring function. For each color component you'll have a function that is color = (multiplicative component) * percent + (additive component) where percent is the percent of completion of the objective."],
					width = "full",
					hidden = hideCustomColorMode,
					order = 6,
				},
				zeroCustomColorFunctionHeader = {
					type = "header",
					name = LCo["Function for objective completion at 0%"],
					width = "full",
					hidden = hideCustomColorMode,
					order = 7,
				},
				rj0 = {
					name = LCo["Red add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 8,
				},
				gj0 = {
					name = LCo["Green add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 9,
				},
				bj0 = {
					name = LCo["Blue add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 10
				},
				firstCustomColorFunctionHeader = {
					type = "header",
					name = LCo["Function for [0%, 50%[ objective completion"],
					hidden = hideCustomColorMode,
					order = 11,
				},
				ri1 = {
					name = LCo["Red mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 12,
				},
				gi1 = {
					name = LCo["Green mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 13,
				},
				bi1 = {
					name = LCo["Blue mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 14,
				},
				rj1 = {
					name = LCo["Red add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 15,
				},
				gj1 = {
					name = LCo["Green add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 16,
				},
				bj1 = {
					name = LCo["Blue add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 17,
				},
				secondCustomColorFunctionHeader = {
					type = "header",
					name = LCo["Function for ]50%, 100%[ objective completion"],
					hidden = hideCustomColorMode,
					order = 18,
				},
				ri2 = {
					name = LCo["Red mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 19,
				},
				gi2 = {
					name = LCo["Green mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 20,
				},
				bi2 = {
					name = LCo["Blue mult."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 21,
				},
				rj2 = {
					name = LCo["Red add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 22,
				},
				gj2 = {
					name = LCo["Green add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 23,
				},
				bj2 = {
					name = LCo["Blue add."],
					type = "range",
					min = -5,
					max = 5,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 24,
				},
				fullCustomColorFunctionHeader = {
					type = "header",
					name = LCo["Function for objective completion at 100%"],
					hidden = hideCustomColorMode,
					order = 25,
				},
				rj3 = {
					name = LCo["Red add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 26,
				},
				gj3 = {
					name = LCo["Green add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 27,
				},
				bj3 = {
					name = LCo["Blue add."],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					hidden = hideCustomColorMode,
					order = 28,
				},
			},
		},
		questStatusColor = {
			type = "group",
			name = LCo["Quest status color"],
			get = hexColorGetter,
			set = hexColorSetter,
			order = 7,
			args = {
				completedColor = {
					name = LCo["Color for quest complete"],
					type = "color",
					hasAlpha = false,
					width = "full",
					order = 1,
				},
				failedColor = {
					name = LCo["Color for quest failed"],
					type = "color",
					hasAlpha = false,
					width = "full",
					order = 2,
				},
			},
		},
		sorting = {
			type = "group",
			name = LCo["Zones and Quests sorting"],
			set = sortSetter,
			order = 8,
			args = {
				zoneSorting = {
					type = "select",
					name = LCo["Zones sorting method"],
					values = zSorting,
					width = "full",
					order = 1,
				},
				zoneWithoutLvlPos = {
					type = "select",
					name = LCo["Position of zones without level when sorting zones by level"],
					set = zwlSetter,
					values = zwlPosition,
					width = "full",
					order = 2,
				},
				questsSorting = {
					type = "select",
					name = LCo["Quests sorting method"],
					values = qSorting,
					width = "full",
					order = 3,
				},
			},
		},
	}
}
local moduleIndex = 0
for k, v in core:IterateModules() do
	moduleIndex = moduleIndex + 1
	options.args.modules.args[k] = {
		type = "toggle",
		name = LCo[k],
		width = "double",
		order = moduleIndex,
	}
end
local raidModuleIndex = 0
for k, v in core:IterateModules() do
	raidModuleIndex = raidModuleIndex + 1
	if k == "achievementTracker" or k == "questsTracker" then
		options.args.raidMode.args[k] = {
			type = "select",
			values = raidModeTrackerValues,
			name = LCo[k],
			width = "full",
			hidden = hideRaidMode,
			order = raidModuleIndex + 2,
		}
	else
		options.args.raidMode.args[k] = {
			type = "select",
			values = raidModeValues,
			name = LCo[k],
			width = "full",
			hidden = hideRaidMode,
			order = raidModuleIndex + 2,
		}
	end
end
if IsAddOnLoaded("ButtonFacade") then
	local LB = LibStub("AceLocale-3.0"):GetLocale("ButtonFacade")
	options.args.buttonFacade = {
		type = "group",
		name = "Button Facade",
		get = getLBF,
		order = 9,
		args = {
			skin = {
				type = "select",
				name = LB["Skin"],
				desc = LB["Set the skin."],
				set = setSkin,
				style = "dropdown",
				width = "full",
				values = core.LBF.ListSkins,
				order = 2,
			},
			gloss = {
				type = "range",
				name = LB["Gloss"],
				desc = LB["Set the intensity of the gloss."],
				set = setGloss,
				min = 0,
				max = 1,
				step = 0.01,
				width = "full",
				isPercent = true,
				order = 3,
			},
			backdrop = {
				type = "toggle",
				name = LB["Backdrop"],
				desc = LB["Toggle the backdrop."],
				set = setBackdrop,
				width = "half",
				order = 4,
			},
			colorHeader = {
				type = "header",
				name = LB["Color Options"],
				order = 5,
			},
			Backdrop = {
				type = "color",
				name = LB["Backdrop"],
				desc = LB["Set the %s color."]:format(LB["Backdrop"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 6,
			},
			Flash = {
				type = "color",
				name = LB["Flash"],
				desc = LB["Set the %s color."]:format(LB["Flash"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 7,
			},
			Normal = {
				type = "color",
				name = LB["Normal"],
				desc = LB["Set the %s color."]:format(LB["Normal"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 8,
			},
			Pushed = {
				type = "color",
				name = LB["Pushed"],
				desc = LB["Set the %s color."]:format(LB["Pushed"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 9,
			},
			Disabled = {
				type = "color",
				name = LB["Disabled"],
				desc = LB["Set the %s color."]:format(LB["Disabled"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 10,
			},
			Checked = {
				type = "color",
				name = LB["Checked"],
				desc = LB["Set the %s color."]:format(LB["Checked"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 11,
			},
			Border = {
				type = "color",
				name = LB["Equipped"],
				desc = LB["Set the %s color."]:format(LB["Equipped"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 12,
			},
			Highlight = {
				type = "color",
				name = LB["Highlight"],
				desc = LB["Set the %s color."]:format(LB["Highlight"]),
				get = getLayerColor,
				set = setLayerColor,
				hasAlpha = true,
				order = 13,
			},
			Gloss = {
				type = "color",
				name = LB["Gloss"],
				desc = LB["Set the %s color."]:format(LB["Flash"]),
				get = getLayerColor,
				set = setLayerColor,
				order = 14,
			},
			resetColors = {
				type = "execute",
				name = LB["Reset Colors"],
				desc = LB["Reset all colors."],
				func = resetColors,
				order = 15,
			},
		},
	}
end
local achievementTrackerOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["achievementTracker"],
	get = achievementOptGetter,
	set = achievementOptSetter,
	args = {
		autotrack = {
			type = "group",
			name = LCo["Auto track"],
			args = {
				timedAchievements = {
					name = LCo["Timed achievements"],
					type = "toggle",
					set = timedAchievementSetter,
					width = "full",
					order = 1,
				},
			},
		},
		display = {
			type = "group",
			name = LCo["Display"],
			args = {
				showInQuestTracker = {
					name = LCo["Show achievements in quests tracker"],
					type = "toggle",
					set = achievementInQuestsTrackerSetter,
					width = "full",
					order = 1,
				},
				positionInQuestTracker = {
					name = LCo["Position of achievements relative to quests in the tracker"],
					type = "select",
					values = achievementPosition,
					set = achievementInQuestsTrackerSetter,
					width = "full",
					order = 2,
				},
				showAnchor = {
					name = LCo["Show achievement tracker anchor"],
					type = "toggle",
					set = achievementAnchorSetter,
					width = "full",
					order = 3,
				},
				anchorPosition = {
					name = LCo["Anchor position"],
					type = "select",
					values = anchorPosition,
					width = "full",
					order = 4,
				},
				toggleTracker = {
					name = LCo["Display tracker"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				hideTrackerInCombat = {
					name = LCo["Hide achievement tracker in combat"],
					type = "toggle",
					set = achievementCombatSetter,
					width = "full",
					order = 6,
				},
				hideCompletedCriteria = {
					name = LCo["Hide completed achievement criteria"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				showByCategory = {
					name = LCo["Display achievements by category and subcategory"],
					type = "toggle",
					set = achievementLayoutSetter,
					width = "full",
					order = 8,
				},
			},
		},
		fonts = {
			type = "group",
			name = LCo["Tracker fonts"],
			set = achievementFontSetter,
			args = {
				category = {
					type = "group",
					name = LCo["Achievement category"],
					inline = true,
					order = 1,
					args = {
						categoryFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						categoryFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						categoryFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				subCategory = {
					type = "group",
					name = LCo["Achievement sub category"],
					inline = true,
					order = 2,
					args = {
						subCategoryFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						subCategoryFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						subCategoryFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				achievement = {
					type = "group",
					name = LCo["Achievement"],
					inline = true,
					order = 3,
					args = {
						achievementFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						achievementFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						achievementFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				achievementCriteria = {
					type = "group",
					name = LCo["Achievement criteria"],
					inline = true,
					order = 4,
					args = {
						achievementCriteriaFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						achievementCriteriaFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						achievementCriteriaFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				statusBar = {
					type = "group",
					name = LCo["Status Bar"],
					inline = true,
					order = 5,
					args = {
						statusBarFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						statusBarFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						statusBarFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
			},
		},
		appearance = {
			type = "group",
			name = LCo["Tracker appearance"],
			args = {
				scale = {
					name = LCo["Achievement tracker scale"],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					width = "full",
					order = 1,
				},
				hasMaxHeight = {
					name = LCo["Set achievements tracker maximum height after which it will be scrollable"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				maxHeight = {
					name = LCo["Achievement tracker max height"],
					type = "range",
					min = 100,
					max = 2000,
					step = 5,
					width = "full",
					order = 3,
				},
				showCollapseButton = {
					name = LCo["Show collapse button"],
					set = achievementLayoutSetter,
					type = "toggle",
					width = "full",
					order = 4,
				},
				categoryCollapseButtonSize = {
					name = LCo["Category collapse Button size"],
					type = "range",
					set = achievementLayoutSetter,
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 5,
				},
				subCategoryCollapseButtonSize = {
					name = LCo["Sub category collapse Button size"],
					type = "range",
					set = achievementLayoutSetter,
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 6,
				},
				displayAchievementImage = {
					name = LCo["Display achievements icon"],
					type = "toggle",
					set = achievementLayoutSetter,
					width = "full",
					order = 7,
				},
				achievementImageSize = {
					name = LCo["Achievement image size"],
					type = "range",
					set = achievementLayoutSetter,
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 8,
				},
				subCategoryCellPadding = {
					name = LCo["Sub category cell padding"],
					type = "range",
					set = achievementLayoutSetter,
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 9,
				},
				achievementCellPadding = {
					name = LCo["Achievement cell padding"],
					type = "range",
					set = achievementLayoutSetter,
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 10,
				},
				criteriaCellPadding = {
					name = LCo["Achievement criteria cell padding"],
					type = "range",
					set = achievementLayoutSetter,
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 11,
				},
				rightCellPadding = {
					name = LCo["Right-most cell right padding to avoid cell clipping"],
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					width = "full",
					order = 12,
				},
				maxWidth = {
					name = LCo["Achievement tracker column max width"],
					type = "range",
					set = achievementLayoutSetter,
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 13,
				},
				minWidth = {
					name = LCo["Achievement tracker column min width"],
					type = "range",
					set = achievementLayoutSetter,
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 14,
				},
				statusMaxWidth = {
					name = LCo["Achievement Tracker status column max width"],
					type = "range",
					set = achievementLayoutSetter,
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 15,
				},
				statusMinWidth = {
					name = LCo["Achievement Tracker status column min width"],
					type = "range",
					set = achievementLayoutSetter,
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 16,
				},
				numBooleanCriteriaColumns = {
					name = LCo["Number of columns in the achievement tracker for boolean criteria"],
					type = "range",
					set = achievementLayoutSetter,
					min = 1,
					max = 8,
					step = 1,
					width = "full",
					order = 17,
				},
			},
		},
		color = {
			type = "group",
			name = LCo["Colors"],
			args = {
				backDropColor = {
					name = LCo["Achievement tracker background color"],
					type = "color",
					hasAlpha = true,
					get = achievementColorGetter,
					set = achievementColorSetter,
					width = "full",
					order = 1,
				},
				borderColor = {
					name = LCo["Achievement tracker border color"],
					type = "color",
					hasAlpha = true,
					get = achievementColorGetter,
					set = achievementColorSetter,
					width = "full",
					order = 2,
				},
				colorizeACName = {
					name = LCo["Color achievements category name"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				defaultACNameColor = {
					name = LCo["Default achievements name color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 4,
				},
				colorizeACCounter = {
					name = LCo["Color achievements category counter"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				defaultACCounterColor = {
					name = LCo["Default achievements category counter color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 6,
				},
				colorizeASCName = {
					name = LCo["Color achievements subcategory name"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				defaultASCNameColor = {
					name = LCo["Default achivements subcategory name color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 8,
				},
				colorizeASCCounter = {
					name = LCo["Color achievements subcategory counter"],
					type = "toggle",
					width = "full",
					order = 9,
				},
				defaultASCCounterColor = {
					name = LCo["Default achievements subcategory counter color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 10,
				},
				colorizeAname = {
					name = LCo["Color achievements name"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				defaultANameColor = {
					name = LCo["Default achievements name color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 12,
				},
				colorizeATimer = {
					name = LCo["Color achievement timer"],
					type = "toggle",
					width = "full",
					order = 13,
				},
				defaultATimerColor = {
					name = LCo["Default achievement timer color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 14,
				},
				colorizeACrName = {
					name = LCo["Color achievements criteria name"],
					type = "toggle",
					width = "full",
					order = 15,
				},
				defaultACrNameColor = {
					name = LCo["Default achievements criteria name color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 16,
				},
				colorizeACrStatus = {
					name = LCo["Color achievements cirteria status"],
					type = "toggle",
					width = "full",
					order = 17,
				},
				defaultACrStatusColor = {
					name = LCo["Default achievements criteria status color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 18,
				},
				colorizeStatusBar = {
					name = LCo["Color status bar"],
					type = "toggle",
					width = "full",
					order = 19,
				},
				defaultStatusBarColor = {
					name = LCo["Default status bar color"],
					type = "color",
					hasAlpha = false,
					get = achievementColorGetter,
					set = achievementColorSetter,
					width = "full",
					order = 20,
				},
				colorizeStatusBarText ={
					name = LCo["Color status bar text"],
					type = "toggle",
					width = "full",
					order = 21,
				},
				defaultStatusBarTextColor = {
					name = LCo["Default status bar text color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 22,
				},
				defaultDescriptionColor = {
					name = LCo["Default achievement description color"],
					type = "color",
					hasAlpha = false,
					get = achievementHexColorGetter,
					set = achievementHexColorSetter,
					width = "full",
					order = 23,
				},
			},
		},
		statusBar = {
			type = "group",
			name = LCo["Status Bar"],
			args = {
				displayAsStatusBar = {
					name = LCo["Display status bar for criteria progression when possible"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				statusBarTexture = {
					name = LCo["Status bar texture"],
					type = "select",
					dialogControl = 'LSM30_Statusbar',
					values = AceGUIWidgetLSMlists.statusbar,
					order = 2,
				},
				statusBarHeight = {
					name = LCo["Status bar height"],
					type = "range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 3,
				},
				leftStatusBarPadding = {
					name = LCo["Left padding of the status bar"],
					type = "range",
					min = 0,
					max = 100,
					step = 1,
					width = "full",
					order = 4,
				},
				rightStatusBarPadding = {
					name = LCo["Right padding of the status bar"],
					type = "range",
					min = 0,
					max = 100,
					step = 1,
					width = "full",
					order = 5,
				},
			},
		},
		interaction = {
			type = "group",
			name = LCo["Tracker interaction"],
			args = {
				disableInteraction = {
					type = "toggle",
					name = LCo["Completely disable tracker interaction, should not be used when display by category is used or if you wish to make it scrollable."],
					set = achievementLayoutSetter,
					width = "full",
					order = 1,
				},
				overTooltip = {
					name = LCo["Display achievement tooltip on mouse over"],
					type = "toggle",
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 2,
				},
				mouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 3,
				},
				mouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 4,
				},
				mouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 5,
				},
				mouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 6,
				},
				mouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 7,
				},
				mouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 8,
				},
				mouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 9,
				},
				mouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 10,
				},
				mouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 11,
				},
				mouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 12,
				},
				mouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 13,
				},
				mouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = achievementInteractivityList,
					width = "full",
					hidden = achievementInteractionDisabled,
					order = 14,
				},
			},
		},
		uiModifications = {
			type = "group",
			name = LCo["UI modifications"],
			args = {
				unitTooltip = {
					type = "toggle",
					name = LCo["Add achievement information to unit tooltip."],
					set = achievementUnitTooltipSetter,
					width = "full",
					order = 1,
				},
				itemTooltip = {
					type = "toggle",
					name = LCo["Add achievement information to item tooltip."],
					set = achievementItemTooltipSetter,
					width = "full",
					order = 2,
				},
				tradeSkillUI = {
					type = "toggle",
					name = LCo["Add achievement information to trade skills UI."],
					set = achievementTradeSkillsIUSetter,
					width = "full",
					order = 3,
				},
			}
		}
	},
}
local announceOptions = {
	type = "group",
	name = LCo["announce"],
	get = announceOptGetter,
	set = announceOptSetter,
	args = {
		playerObjectiveProgress = {
			name = LCo["Quests objectives progress annoucement"],
			type = "toggle",
			width = "full",
			order = 12,
		},
		playerObjectiveComplete = {
			name = LCo["Quests objectives complete announcement"],
			type = "toggle",
			width = "full",
			order = 13,
		},
		playerQuestComplete = {
			name = LCo["Quests complete announcement"],
			type = "toggle",
			width = "full",
			order = 14,
		},
		playerQuestFailed = {
			name = LCo["Quests failed announcement"],
			type = "toggle",
			width = "full",
			order = 15,
		},
	},
}
local dailyTooltipOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["dailyTooltip"],
	get = dailyTooltipOptGetter,
	set = dailyTooltipOptSetter,
	args = {
		counter = {
			type = "group",
			name = LCo["Daily quests counter"],
			set = dailyTooltipDailySetter,
			args = {
				showNumCompletedDailyQuests = {
					name = LCo["Show number of completed daily quest"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				showMaxDailyQuest = {
					name = LCo["Show maximum daily quests number"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				showRemainingTime = {
					name = LCo["Show remaining time before end of day"],
					type = "toggle",
					set = dailyTooltipTimeSetter,
					width = "full",
					order = 3,
				},
				colorizeDailyQuestsCounter = {
					name = LCo["Color daily quests counter"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				colorizeDailyQuestsTimer = {
					name = LCo["Color daily quests timer"],
					type = "toggle",
					width = "full",
					order = 5,
				},
			},
		},
		tooltip = {
			type = "group",
			name = LCo["dailyTooltip"],
			args = {
				showCOTD = {
					name = LCo["Show in the tooltip daily quests completed today"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				dqFont = {
					type = "group",
					name = LCo["Font"],
					set = dailyTooltipFontSetter,
					inline = true,
					order = 2,
					args = {
						font = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						fontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						fontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
			},
		},
		ldbInteraction = {
			type = "group",
			name = LCo["Daily quests counter interaction"],
			args = {
				ldbMouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 1,
				},
				ldbMouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 2,
				},
				ldbMouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 3,
				},
				ldbMouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 4,
				},
				ldbMouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 5,
				},
				ldbMouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 6,
				},
				ldbMouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 7,
				},
				ldbMouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 8,
				},
				ldbMouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 9,
				},
				ldbMouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 10,
				},
				ldbMouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 11,
				},
				ldbMouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 12,
				},
			},
		},
	},
}
local notificationsOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["notifications"],
	get = notificationsOptGetter,
	set = notificationsOptSetter,
	args = {
		common = {
			type = "group",
			name = LCo["Common settings"],
			args = {
				hideBlizzardNotification = {
					name = LCo["Hide default blizzard quests notification"],
					type = "toggle",
					set = notificationsHideBlizzardNotificationSetter,
					width = "full",
					order = 1,
				},
				objectiveNotificationWhenComplete = {
					name = LCo["Objective notification on quest completion"],
					type = "toggle",
					width = "full",
					order = 2,
				},
			},
		},
		player = {
			type = "group",
			name = LCo["Player Notifications"],
			args = {
				playerObjectiveProgress = {
					name = LCo["Quest objective progress notification"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				playerObjectiveComplete = {
					name = LCo["Quest objective complete notification"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				playerQuestComplete = {
					name = LCo["Quest complete notification"],
					type = "toggle",
					set = notificationsCallbackSetter,
					width = "full",
					order = 3,
				},
				playerQuestFailed = {
					name = LCo["Quest failed notification"],
					type = "toggle",
					set = notificationsCallbackSetter,
					width = "full",
					order = 4,
				},
				itemStartingQuestLooted = {
					name = LCo["Item starting a quest notification"],
					type = "toggle",
					set = notificationsCallbackSetter,
					width = "full",
					order = 5,
				},
			},
		},
		party = {
			type = "group",
			name = LCo["Party Notifications"],
			args = {
				partyObjectiveProgress = {
					name = LCo["Quest objective progress notification"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				partyObjectiveComplete = {
					name = LCo["Quest objective complete notification"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				partyQuestComplete = {
					name = LCo["Quest complete notification"],
					type = "toggle",
					set = notificationsCallbackSetter,
					width = "full",
					order = 3,
				},
				partyQuestFailed = {
					name = LCo["Quest failed notification"],
					type = "toggle",
					set = notificationsCallbackSetter,
					width = "full",
					order = 4,
				},
				partyQuestAccepted = {
					name = LCo["Quest accepted notification"],
					type = "toggle",
					set = notificationsCallbackSetter,
					width = "full",
					order = 5,
				},
				partyQuestTurnedIn = {
					name = LCo["Quest turned in notification"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				partyQuestAbandonned = {
					name = LCo["Quest abandonned notification"],
					type = "toggle",
					width = "full",
					order = 7,
				},
			},
		},
		output = notifications:GetSinkAce3OptionsDataTable()
	},
}
local questsGiversOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["questsGivers"],
	get = questsGiversOptGetter,
	set = questsGiversOptSetter,
	args = {
		addQuestLevel = {
			name = LCo["Add quest level to the quest name"],
			type = "toggle",
			width = "full",
			order = 1,
		},
		fadeNotCompletedQuests = {
			name = LCo["Fade quest name that are not complete"],
			type = "toggle",
			width = "full",
			order = 2,
		},
		automaticAccept = {
			name = LCo["Accept quests automatically"],
			type = "select",
			values = automaticAcceptChoices,
			width = "full",
			order = 3,
		},
		automaticAcceptEscort = {
			name = LCo["Accept escort quest started by other player automatically"],
			type = "toggle",
			width = "full",
			order = 4,
		},
		automaticTurnIn = {
			name = LCo["Automatic quest turn-in"],
			type = "toggle",
			width = "full",
			order = 5,
		},
	},
}
local questsInformationsOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["informations"],
	get = questsInformationsOptGetter,
	set = questsInformationsOptSetter,
	args = {
		common = {
			type = "group",
			name = LCo["Common settings"],
			args = {
				backDropColor = {
					name = LCo["Background color"],
					type = "color",
					hasAlpha = true,
					get = questsInformationsColorGetter,
					set = questsInformationsColorSetter,
					width = "full",
					order = 1,
				},
				borderColor = {
					name = LCo["Border color"],
					type = "color",
					hasAlpha = true,
					get = questsInformationsColorGetter,
					set = questsInformationsColorSetter,
					width = "full",
					order = 2,
				},
				attach = {
					name = LCo["Attach LightHeaded to QuestLog"],
					type = "select",
					values = attachSelect,
					set = questsInformationsFrameSetter,
					width = "full",
					order = 3,
				},
			},
		},
		questLog = {
			type = "group",
			name = LCo["Quests log settings"],
			args = {
				showQuestLog = {
					name = LCo["Show tomQuest2 quest log"],
					type = "toggle",
					set = questsInformationsFrameSetter,
					width = "full",
					order = 1,
				},
				qlFont = {
					name = LCo["Font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					width = "full",
					order = 2,
				},
				qlFontSizeH1 = {
					name = LCo["Quest title font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 3,
				},
				qlFontSizeH2 = {
					name = LCo["Headers font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 4,
				},
				qlFontSize = {
					name = LCo["text font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 5,
				},
			},
		},
		lightHeaded = {
			type = "group",
			name = LCo["LightHeaded settings"],
			args = {
				showLightHeaded = {
					name = LCo["Show LightHeaded into tomQuest2"],
					type = "toggle",
					set = questsInformationsFrameSetter,
					width = "full",
					order = 1,
				},
				showAllCommentsOnOnePage = {
					name = LCo["Show all LightHeaded comments on the same page"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				lhFont = {
					name = LCo["Font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					width = "full",
					order = 3,
				},
				lhFontSize = {
					name = LCo["Font size"],
					type="range",
					min = 6,
					max = 20,
					step = 1,
					width = "full",
					order = 4,
				},
			},
		},
	},
}
local questsTooltipOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["questsTooltip"],
	get = questsTooltipOptGetter,
	set = questsTooltipOptSetter,
	args = {
		questsCounter = {
			type = "group",
			name = LCo["Quests counter"],
			set = questsTooltipLdbTextSetter,
			args = {
				showNumCompletedQuests = {
					name = LCo["Show number of completed quest"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				showNumQuests = {
					name = LCo["Show number of quests"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				showMaxQuest = {
					name = LCo["Show maximum quests number"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				colorizeQuestsCounter = {
					name = LCo["Color quests counter"],
					type = "toggle",
					width = "full",
					order = 4,
				},
			},
		},
		display = {
			type = "group",
			name = LCo["Display"],
			args = {
				hideCompletedObjective = {
					name = LCo["Hide completed quests Objectives"],
					type = "select",
					values = hideObjectives,
					width = "full",
					order = 1,
				},
				hideZoneLevelRange = {
					name = LCo["Hide zone level range"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				hideQuestTag = {
					name = LCo["Hide quest tag"],
					type = "toggle",
					width = "full",
					order = 3,
				},
			},
		},
		fonts = {
			type = "group",
			name = LCo["Tooltip fonts"],
			set = questsTooltipFontSetter,
			args = {
				zone = {
					type = "group",
					name = LCo["Zone"],
					inline = true,
					order = 1,
					args = {
						headerFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						headerFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						headerFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				quests = {
					type = "group",
					name = LCo["Quests"],
					inline = true,
					order = 2,
					args = {
						questFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						questFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						questFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				questsObjectives = {
					type = "group",
					name = LCo["Quests objectives"],
					inline = true,
					order = 3,
					args = {
						questObjectiveFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						questObjectiveFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						questObjectiveFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
			},
		},
		appearance = {
			type = "group",
			name = LCo["Tooltip appearance"],
			args = {
				scale = {
					name = LCo["Quests Tooltip scale"],
					type="range",
					min = 0,
					max = 1,
					step = 0.1,
					width = "full",
					order = 1,
				},
				hasMaxHeight = {
					name = LCo["Set quests tooltip maximum height after which it will be scrollable"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				maxHeight = {
					name = LCo["Quests tooltip max height"],
					type = "range",
					min = 100,
					max = 2000,
					step = 5,
					width = "full",
					order = 3,
				},
				showCollapseButton = {
					name = LCo["Show collapse button"],
					set = questsTooltipLayoutSetter,
					type = "toggle",
					width = "full",
					order = 4,
				},
				collapseButtonSize = {
					name = LCo["Collapse Button Size"],
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 5,
				},
				questsCellPadding = {
					name = LCo["Quests cell padding"],
					type="range",
					set = questsTooltipLayoutSetter,
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 6,
				},
				objectivesCellPadding = {
					name = LCo["Quests objectives cell padding"],
					type="range",
					set = questsTooltipLayoutSetter,
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 7,
				},
				rightCellPadding = {
					name = LCo["Right-most cell right padding to avoid cell clipping"],
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					width = "full",
					order = 8,
				},
				maxWidth = {
					name = LCo["Quests Tooltip column max width"],
					type="range",
					set = questsTooltipLayoutSetter,
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 9,
				},
				minWidth = {
					name = LCo["Quests Tooltip column min width"],
					type="range",
					set = questsTooltipLayoutSetter,
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 10,
				},
				statusMaxWidth = {
					name = LCo["Quests Tooltip status column max width"],
					type="range",
					set = questsTooltipLayoutSetter,
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 11,
				},
				statusMinWidth = {
					name = LCo["Quests Tooltip status column min width"],
					type="range",
					set = questsTooltipLayoutSetter,
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 12,
				},
			},
		},
		colors = {
			type = "group",
			name = LCo["Colors"],
			args = {
				colorizeHeaderName = {
					name = LCo["Color quests header name"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				defaultHeaderNameColor = {
					name = LCo["Default header name color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 2,
				},
				colorizeHeaderLevel = {
					name = LCo["Color quests header level"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				defaultHeaderLevelColor = {
					name = LCo["Default header level color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 4,
				},
				colorizeQuestName = {
					name = LCo["Color quests name"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				defaultQuestNameColor = {
					name = LCo["Default quest name color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 6,
				},
				colorizeQuestLevel = {
					name = LCo["Color quests level"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				defaultQuestLevelColor = {
					name = LCo["Default quest level color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 8,
				},
				colorizeQuestObjective = {
					name = LCo["Color quests objectives"],
					type = "toggle",
					width = "full",
					order = 9,
				},
				defaultQuestObjectiveColor = {
					name = LCo["Default quest objective color"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 10,
				},
				colorizeQuestObjectiveCounter = {
					name = LCo["Color quests objectives counter"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				defaultQuestDescriptionColor = {
					name = LCo["Default quest description color for quest without objective"],
					type = "color",
					hasAlpha = false,
					get = questsTooltipHexColorGetter,
					set = questsTooltipHexColorSetter,
					width = "full",
					order = 12,
				},
			},
		},
		questsItems = {
			type = "group",
			name = LCo["Quests items"],
			args = {
				showQuestItemIcon = {
					name = LCo["Show special quest item icon behind quest name"],
					type = "toggle",
					set = questsTooltipLayoutSetter,
					width = "full",
					order = 1,
				},
				questItemIconSize = {
					name = LCo["Quest item icon size"],
					type = "range",
					set = questsTooltipLayoutSetter,
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 2,
				},
			},
		},
		tooltipInteraction = {
			type = "group",
			name = LCo["Tooltip interaction"],
			args = {
				showHints = {
					name = LCo["Show interractivity hints at the bottom of the tooltip"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				overTooltip = {
					name = LCo["Display quest tooltip on mouse over"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				mouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 3,
				},
				mouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 4,
				},
				mouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 5,
				},
				mouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 6,
				},
				mouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 7,
				},
				mouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 8,
				},
				mouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 9,
				},
				mouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 10,
				},
				mouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 11,
				},
				mouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 12,
				},
				mouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 13,
				},
				mouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					order = 14,
				},
			},
		},
		ldbInteraction = {
			type = "group",
			name = LCo["Quests counter interaction"],
			args = {
				ldbMouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 1,
				},
				ldbMouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 2,
				},
				ldbMouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 3,
				},
				ldbMouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 4,
				},
				ldbMouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 5,
				},
				ldbMouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 6,
				},
				ldbMouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 7,
				},
				ldbMouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 8,
				},
				ldbMouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 9,
				},
				ldbMouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 10,
				},
				ldbMouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 11,
				},
				ldbMouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = ldbInteractivityList,
					width = "full",
					order = 12,
				},
			},
		},
	},
}
local questsTrackerOptions = {
	type = "group",
	childGroups = "tab",
	name = LCo["questsTracker"],
	get = questsTrackerOptGetter,
	set = questsTrackerOptSetter,
	args = {
		autotrack = {
			type = "group",
			name = LCo["Auto track"],
			args = {
				onLogIn = {
					name = LCo["Track all quests on log in"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				onQuestAccepted = {
					name = LCo["Auto track quests when you accept them"],
					type = "select",
					values = questsAutoTrack,
					width = "full",
					order = 2,
				},
				onObjectiveProgress = {
					name = LCo["Auto track quests on objective progress"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				unTrackInactive = {
					name = LCo["Auto untrack inactive quests that were tracked on objective progress"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				unTrackDelay = {
					name = LCo["Auto untrack delay (in minutes)"],
					type = "range",
					min = 1,
					max = 30,
					step = 0.5,
					width = "full",
					order = 5,
				},
				onSubZone = {
					name = LCo["Auto track quests on sub zone change"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				onZone = {
					name = LCo["Auto track quests on zone change"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				onDaily = {
					name = LCo["Auto track daily quests"],
					type = "toggle",
					width = "full",
					order = 8,
				},
				onClass = {
					name = LCo["Auto track class quests"],
					type = "toggle",
					width = "full",
					order = 9,
				},
			},
		},
		display = {
			type = "group",
			name = LCo["Display"],
			args = {
				showAnchor = {
					name = LCo["Show quest tracker anchor"],
					type = "toggle",
					width = "full",
					set = questsTrackerAnchorSetter,
					order = 1,
				},
				anchorPosition = {
					name = LCo["Anchor position"],
					type = "select",
					values = anchorPosition,
					width = "full",
					order = 2,
				},
				displayHeader = {
					name = LCo["Display quests by zone"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				autoCollapseHeader = {
					name = LCo["Automatically expand /collapse zone header on zone change"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				toggleTracker = {
					name = LCo["Display tracker"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				hideTrackerInCombat = {
					name = LCo["Hide quest tracker in combat"],
					type = "toggle",
					set = questsTrackerCombatSetter,
					width = "full",
					order = 6,
				},
				hideCompletedQuest = {
					name = LCo["Hide completed quest"],
					type = "select",
					values = hideQuests,
					width = "full",
					order = 7,
				},
				delayToHideCompletedQuest = {
					name = LCo["Delay to hide completed quest"],
					type = "range",
					min = 0,
					max = 60,
					step = 0.5,
					width = "full",
					order = 8,
				},
				hideCompletedObjective = {
					name = LCo["Hide completed quests Objectives"],
					type = "select",
					values = hideObjectives,
					width = "full",
					order = 9,
				},
				onlyCurrentZone = {
					name = LCo["Hide quests that are not in the current zone"],
					type = "toggle",
					width = "full",
					order = 10,
				},
				limitQuestLevel = {
					name = LCo["Hide high level quests (You'll have to choose the maximum level difference)"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				maxLevelDiff = {
					name = LCo["Maximum level difference between the weighted quest level and your level"],
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					get = questsTrackerMaxLevelDifferenceGetter,
					set = questsTrackerMaxLevelDifferenceSetter,
					width = "full",
					order = 12,
				},
				hideZoneLevelRange = {
					name = LCo["Hide zone level range"],
					type = "toggle",
					width = "full",
					order = 13,
				},
				hideQuestTag = {
					name = LCo["Hide quest tag"],
					type = "toggle",
					width = "full",
					order = 14,
				},
			},
		},
		fonts = {
			type = "group",
			name = LCo["Tracker fonts"],
			set = questsTrackerFontSetter,
			args = {
				zone = {
					type = "group",
					name = LCo["Zone"],
					inline = true,
					order = 1,
					args = {
						headerFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						headerFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						headerFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				quests = {
					type = "group",
					name = LCo["Quests"],
					inline = true,
					order = 2,
					args = {
						questFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						questFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						questFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
				questsObjectives = {
					type = "group",
					name = LCo["Quests objectives"],
					inline = true,
					order = 3,
					args = {
						questObjectiveFont = {
							name = LCo["Font"],
							type = "select",
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							width = "full",
							order = 1,
						},
						questObjectiveFontSize = {
							name = LCo["Font size"],
							type = "range",
							min = 6,
							max = 20,
							step = 1,
							width = "full",
							order = 2,
						},
						questObjectiveFontOutline = {
							name = LCo["Font outline"],
							type = "select",
							values = outlines,
							width = "full",
							order = 3,
						},
					},
				},
			},
		},
		appearance = {
			type = "group",
			name = LCo["Tracker appearance"],
			args = {
				scale = {
					name = LCo["Quests Tracker scale"],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					width = "full",
					order = 1,
				},
				hasMaxHeight = {
					name = LCo["Set quests tracker maximum height after which it will be scrollable"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				maxHeight = {
					name = LCo["Quests tracker max height"],
					type = "range",
					min = 100,
					max = 2000,
					step = 5,
					width = "full",
					order = 3,
				},
				showCollapseButton = {
					name = LCo["Show collapse button"],
					set = questsTrackerLayoutSetter,
					type = "toggle",
					width = "full",
					order = 4,
				},
				collapseButtonSize = {
					name = LCo["Collapse Button Size"],
					set = questsTrackerLayoutSetter,
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 5,
				},
				questsCellPadding = {
					name = LCo["Quests cell padding"],
					type = "range",
					set = questsTrackerLayoutSetter,
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 6,
				},
				objectivesCellPadding = {
					name = LCo["Quests objectives cell padding"],
					type = "range",
					set = questsTrackerLayoutSetter,
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 7,
				},
				rightCellPadding = {
					name = LCo["Right-most cell right padding to avoid cell clipping"],
					type = "range",
					set = questsTrackerLayoutSetter,
					min = 0,
					max = 10,
					step = 1,
					width = "full",
					order = 8,
				},
				maxWidth = {
					name = LCo["Quests Tracker column max width"],
					type = "range",
					set = questsTrackerLayoutSetter,
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 9,
				},
				minWidth = {
					name = LCo["Quests Tracker column min width"],
					type = "range",
					set = questsTrackerLayoutSetter,
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 10,
				},
				statusMaxWidth = {
					name = LCo["Quests Tracker status column max width"],
					type="range",
					set = questsTrackerLayoutSetter,
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 11,
				},
				statusMinWidth = {
					name = LCo["Quests Tracker status column min width"],
					type="range",
					set = questsTrackerLayoutSetter,
					min = 30,
					max = 200,
					step = 5,
					width = "full",
					order = 12,
				},
			},
		},
		color = {
			type = "group",
			name = LCo["Colors"],
			args = {
				backDropColor = {
					name = LCo["Quests Tracker background color"],
					type = "color",
					hasAlpha = true,
					get = questsTrackerColorGetter,
					set = questsTrackerColorSetter,
					width = "full",
					order = 1,
				},
				borderColor = {
					name = LCo["Quests Tracker border color"],
					type = "color",
					hasAlpha = true,
					get = questsTrackerColorGetter,
					set = questsTrackerColorSetter,
					width = "full",
					order = 2,
				},
				colorizeHeaderName = {
					name = LCo["Color quests header name"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				defaultHeaderNameColor = {
					name = LCo["Default header name color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 4,
				},
				colorizeHeaderLevel = {
					name = LCo["Color quests header level"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				defaultHeaderLevelColor = {
					name = LCo["Default header level color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 6,
				},
				colorizeQuestName = {
					name = LCo["Color quests name"],
					type = "toggle",
					width = "full",
					order = 7,
				},
				defaultQuestNameColor = {
					name = LCo["Default quest name color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 8,
				},
				colorizeQuestLevel = {
					name = LCo["Color quests level"],
					type = "toggle",
					width = "full",
					order = 9,
				},
				defaultQuestLevelColor = {
					name = LCo["Default quest level color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 10,
				},
				colorizeQuestTimer = {
					name = LCo["Color quests timer"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				defaultQuestTimerColor = {
					name = LCo["Default quest timer color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 12,
				},
				colorizeQuestObjective = {
					name = LCo["Color quests objectives"],
					type = "toggle",
					width = "full",
					order = 13,
				},
				defaultQuestObjectiveColor = {
					name = LCo["Default quest objective color"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 14,
				},
				colorizeQuestObjectiveCounter = {
					name = LCo["Color quests objectives counter"],
					type = "toggle",
					width = "full",
					order = 15,
				},
				defaultQuestDescriptionColor = {
					name = LCo["Default quest description color for quest without objective"],
					type = "color",
					hasAlpha = false,
					get = questsTrackerHexColorGetter,
					set = questsTrackerHexColorSetter,
					width = "full",
					order = 16,
				},
			},
		},
		questsItems = {
			type = "group",
			name = LCo["Quests items"],
			args = {
				settings = {
					type = "group",
					name = LCo["Settings"],
					inline = true,
					order = 1,
					args = {
						showQuestItemIcon = {
							name = LCo["Show special quest item icon behind quest name"],
							type = "toggle",
							set = questsTrackerLayoutSetter,
							width = "full",
							order = 1,
						},
						questItemIconSize = {
							name = LCo["Quest item icon size"],
							type = "range",
							set = questsTrackerLayoutSetter,
							min = 6,
							max = 36,
							step = 1,
							width = "full",
							order = 2,
						},
						oorColor = {
							name = LCo["Out of range color"],
							type = "color",
							hasAlpha = false,
							get = questsTrackerColorGetter,
							set = questsTrackerColorSetter,
							width = "full",
							order = 3,
						},
						showItemsStartingQuest = {
							name = LCo["Show items starting a quest in the tracker"],
							type = "toggle",
							width = "full",
							order = 4,
						},
						showUsableQuestItemsNotAttached = {
							name = LCo["Show usable quest item not linked to a quest"],
							type = "toggle",
							width = "full",
							order = 5,
						},
						otherItemIconSize = {
							name = LCo["Other quest item icon size"],
							type = "range",
							min = 6,
							max = 36,
							step = 1,
							width = "full",
							order = 6,
						},
					},
				},
			},
		},
		trackerInteraction = {
			type = "group",
			name = LCo["Tracker interaction"],
			args = {
				disableInteraction = {
					type = "toggle",
					name = LCo["Completely disable tracker interaction, should not be used when display by category is used or if you wish to make it scrollable."],
					set = questsTrackerLayoutSetter,
					width = "full",
					order = 1,
				},
				overTooltip = {
					name = LCo["Display quest tooltip on mouse over"],
					type = "toggle",
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 2,
				},
				mouseRightShift = {
					name = LCo["Action on right shift click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 3,
				},
				mouseRightControl = {
					name = LCo["Action on right control click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 4,
				},
				mouseRightAlt = {
					name = LCo["Action on right alt click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 5,
				},
				mouseRight = {
					name = LCo["Action on right click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 6,
				},
				mouseLeftShift = {
					name = LCo["Action on left shift click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 7,
				},
				mouseLeftControl = {
					name = LCo["Action on left control click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 8,
				},
				mouseLeftAlt = {
					name = LCo["Action on left alt click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 9,
				},
				mouseLeft = {
					name = LCo["Action on left click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 10,
				},
				mouseMiddleShift = {
					name = LCo["Action on middle shift click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 11,
				},
				mouseMiddleControl = {
					name = LCo["Action on middle control click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 12,
				},
				mouseMiddleAlt = {
					name = LCo["Action on middle alt click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 13,
				},
				mouseMiddle = {
					name = LCo["Action on middle click"],
					type = "select",
					values = questsInteractivityList,
					width = "full",
					hidden = questsTrackerInteractionDisabled,
					order = 14,
				},
			},
		},
	},
}
local soundNotificationsOptions = {
	type = "group",
	name = LCo["soundNotifications"],
	get = soundNotificationsOptGetter,
	set = soundNotificationsOptSetter,
	args = {
		soundObjectiveComplete = {
			name = LCo["Quest objective complete sound"],
			type = "select",
			dialogControl = 'LSM30_Sound',
			values = AceGUIWidgetLSMlists.sound,
			width = "full",
			order = 1,
		},
		soundQuestComplete = {
			name = LCo["Quest complete sound"],
			type = "select",
			dialogControl = 'LSM30_Sound',
			values = AceGUIWidgetLSMlists.sound,
			width = "full",
			order = 2,
		},
		soundQuestFailed = {
			name = LCo["Quest failed sound"],
			type = "select",
			dialogControl = 'LSM30_Sound',
			values = AceGUIWidgetLSMlists.sound,
			width = "full",
			order = 3,
		},
	},
}
local tooltipsOptions = {
	type = "group",
	name = LCo["tooltips"],
	get = tooltipOptGetter,
	set = tooltipOptSetter,
	args = {
		mobTooltip = {
			name = LCo["Mobs tooltip modification"],
			type = "toggle",
			width = "full",
			order = 10,
		},
		itemTooltip = {
			name = LCo["Item tooltip modification"],
			type = "toggle",
			width = "full",
			order = 12,
		},
	},
}
local profilesOptions = LibStub("AceDBOptions-3.0"):GetOptionsTable(core.db)

ACR:RegisterOptionsTable("tomQuest2", options)
core.mainOptionFrame = ACD:AddToBlizOptions("tomQuest2", "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_profiles", profilesOptions)
core.profilesOptionFrame = ACD:AddToBlizOptions("tomQuest2_profiles", LCo["Profiles"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_achievementTracker", achievementTrackerOptions)
ACD:AddToBlizOptions("tomQuest2_achievementTracker", LCo["achievementTracker"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_announce", announceOptions)
ACD:AddToBlizOptions("tomQuest2_announce", LCo["announce"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_dailyTooltip", dailyTooltipOptions)
ACD:AddToBlizOptions("tomQuest2_dailyTooltip", LCo["dailyTooltip"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_notifications", notificationsOptions)
ACD:AddToBlizOptions("tomQuest2_notifications", LCo["notifications"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsGivers", questsGiversOptions)
ACD:AddToBlizOptions("tomQuest2_questsGivers", LCo["questsGivers"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsInformations", questsInformationsOptions)
ACD:AddToBlizOptions("tomQuest2_questsInformations", LCo["informations"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsTooltip", questsTooltipOptions)
ACD:AddToBlizOptions("tomQuest2_questsTooltip", LCo["questsTooltip"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsTracker", questsTrackerOptions)
ACD:AddToBlizOptions("tomQuest2_questsTracker", LCo["questsTracker"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_soundNotifications", soundNotificationsOptions)
ACD:AddToBlizOptions("tomQuest2_soundNotifications", LCo["soundNotifications"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_tooltips", tooltipsOptions)
ACD:AddToBlizOptions("tomQuest2_tooltips", LCo["tooltips"], "tomQuest2")